package com.alihealth.d2d.provtest.controller;

import com.alibaba.fastjson.JSON;
import com.alihealth.d2d.provtest.domain.ReceiveDataLog;
import com.alihealth.d2d.provtest.enums.BuzStatusEnum;
import com.alihealth.d2d.provtest.enums.EventTypeEnum;
import com.alihealth.d2d.provtest.enums.SubTypeEnum;
import com.alihealth.d2d.provtest.service.*;
import com.alihealth.d2d.provtest.utils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Map;

/**
 * @ClassName ProvTestServerController
 * @Desc 省局对接示例代码  服务端  接收端
 * @Author houduo.wk
 * @Date 2020/3/13
 **/
@RestController
@RequestMapping("/prov/test")
public class ProvTestServerController {

    Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private CertManager certManager;

    @Autowired
    private IReceiveDataLogService receiveDataLogService;


    @Autowired
    private IEventProcessingService processingService;

    @RequestMapping("/receive")
    public @ResponseBody
    Map<String,Object> receive(HttpServletRequest request) throws IOException {
        RequestWrapper requestWrapper = new RequestWrapper(request);
        String body = requestWrapper.getBodyString();
        System.out.println(body);
        Map<String,Object> paramsTemp = JSON.parseObject(body);
        System.out.println(paramsTemp);
        Map<String,Object> params = JSON.parseObject(paramsTemp.get("req").toString());
        logger.info("ProvTestController.receive.in.params:{}",params);
        try{
            //传输流水号0
            String processId = String.valueOf(params.get("process_id"));
            //事件编号
            String eventId = String.valueOf(params.get("event_id"));
            //主数据类型
            String eventType = String.valueOf(params.get("event_type"));
            //主数据 包含子类型sub_type和数据data
            String event = String.valueOf(params.get("event"));
            //时间戳 YYMMddHHmmssSSS
            String timeStamp = String.valueOf(params.get("timestamp"));
            //版本号
            String ver = String.valueOf(params.get("ver"));

            logger.info("processId:{}|eventId:{}|eventType:{}|event:{}|timeStamp:{}|ver:{}"
                    ,processId,eventId,eventType,event,timeStamp,ver);

            EventPojo eventPojo = JSON.parseObject(event, EventPojo.class);
            //获取私钥
            String privateKey = certManager.getPrivateKey();

            logger.info("privateKey:{}",privateKey);


            String de = certManager.testDecrypt(privateKey,eventPojo.getData());
            String xml = StreamUtil.getUnStreamContent(de);
            logger.info("解密之后的文件内容:{}", xml);

            //保存数据日志
            ReceiveDataLog log=new ReceiveDataLog();
            log.setEventId(eventId);
            log.setProcessId(processId);
            log.setEventType(eventType);
            log.setEventTypeName(EventTypeEnum.getNameByType(eventType));
            log.setSubType(eventPojo.getSubType());
            log.setSubTypeName(SubTypeEnum.getNameByType(eventPojo.getSubType()));
            log.setData(eventPojo.getData());
            log.setVer(ver);
            receiveDataLogService.save(log);

            //解析并处理数据
            processingService.processEvent(eventPojo, xml,eventId);

            return ResponseBuilder.success();
        }catch (Exception e){
            logger.error("{}",e);
            return ResponseBuilder.fail(BuzStatusEnum.SYS_ERR);
        }

    }



}
